package misc;

import bsh.EvalError;
import bsh.Interpreter;
import gg.xp.xivsupport.callouts.CalloutTests;
import groovy.lang.Closure;
import groovy.lang.GString;
import groovy.lang.GroovyShell;
import groovy.lang.Script;
import jdk.jshell.JShell;
import jdk.jshell.execution.LocalExecutionControlProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

import javax.script.ScriptEngineManager;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.zip.GZIPOutputStream;

//@Ignore
@SuppressWarnings("ResultOfObjectAllocationIgnored")
public class Benchmarks {

	private static final Logger log = LoggerFactory.getLogger(Benchmarks.class);

	private static final String logline = "38|2021-05-10T22:31:52.3330000-07:00|10669D22|Wynn Dohz|0050501C|122634|122634|10000|10000|27|0|80.35929|4.77615|17.09997|0.869517|3A00|AA|0|0A016D|41F00000|E0000000|0A0168|41F00000|E0000000||c243843d37798b12a00ec8cef4fd4d43";

	private static void timeIt(String label, Runnable run) {
		long before = System.currentTimeMillis();
		run.run();
		long after = System.currentTimeMillis();
		long delta = after - before;
		log.info("Timing for {}: {}", label, delta);
	}

	@Test
	void testBasicMatch() {
		int count = 10_000_000;
		// Just in case a field access is slower than a local var, copy it here
		String line = logline;
		Pattern m38 = Pattern.compile("^38\\|.*");
		Pattern m21 = Pattern.compile("^21\\|.*");
		Pattern f38 = Pattern.compile("^38\\|");
		Pattern f21 = Pattern.compile("^21\\|");
		String s38 = "38|";
		String s21 = "21|";

		// Verify patterns
		Assert.assertTrue(m38.matcher(line).matches());
		Assert.assertFalse(m21.matcher(line).matches());
		Assert.assertTrue(f38.matcher(line).find());
		Assert.assertFalse(f21.matcher(line).find());
		Assert.assertTrue(line.startsWith(s38));
		Assert.assertFalse(line.startsWith(s21));

		// using matches()
		timeIt("matching pattern using matches", () -> {
			for (int i = 0; i < count; i++) {
				m38.matcher(line).matches();
			}
		});
		timeIt("non-matching pattern using matches", () -> {
			for (int i = 0; i < count; i++) {
				m21.matcher(line).matches();
			}
		});

		// using find()
		timeIt("matching pattern using find", () -> {
			for (int i = 0; i < count; i++) {
				f38.matcher(line).find();
			}
		});
		timeIt("non-matching pattern using find", () -> {
			for (int i = 0; i < count; i++) {
				f21.matcher(line).find();
			}
		});

		// using startsWith()
		// using find()
		timeIt("matching pattern using startsWith", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
			}
		});
		timeIt("non-matching pattern using startsWith", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s21);
			}
		});
	}

	@Test
	void testCapturing() {
		int count = 1_000_000;
		// Just in case a field access is slower than a local var, copy it here
		String line = logline;
		Pattern m38 = Pattern.compile("^38\\|[^|]*\\|([^|]*)\\|.*");
		Pattern m21 = Pattern.compile("^21\\|[^|]*\\|([^|]*)\\|.*");
		Pattern f38 = Pattern.compile("^38\\|[^|]*\\|([^|]*)");
		Pattern f21 = Pattern.compile("^21\\|[^|]*\\|([^|]*)");
		String s38 = "38|";
		String s21 = "21|";

		// Verify patterns
		Assert.assertTrue(m38.matcher(line).matches());
		Assert.assertFalse(m21.matcher(line).matches());
		Assert.assertTrue(f38.matcher(line).find());
		Assert.assertFalse(f21.matcher(line).find());
		Assert.assertTrue(line.startsWith(s38));
		Assert.assertFalse(line.startsWith(s21));

		// using matches()
		timeIt("matching pattern using matches", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = m38.matcher(line);
				matcher.matches();
				matcher.group(1);
			}
		});
		timeIt("non-matching pattern using matches", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = m21.matcher(line);
				matcher.matches();
			}
		});

		// using find()
		timeIt("matching pattern using find", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = f38.matcher(line);
				matcher.find();
				matcher.group(1);
			}
		});
		timeIt("non-matching pattern using find", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = f21.matcher(line);
				matcher.find();
			}
		});

		// using startsWith()
		timeIt("matching pattern using startsWith", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
				line.split("\\|", 4);
			}
		});
		timeIt("matching pattern using startsWith (increased limit)", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
				line.split("\\|", 12);
			}
		});
		timeIt("matching pattern using startsWith (no limit)", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
				line.split("\\|");
			}
		});
		timeIt("non-matching pattern using startsWith", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s21);
			}
		});

	}

	@Test
	void testConditions() {
		// TODO
		int count = 1_000_000;
		// Just in case a field access is slower than a local var, copy it here
		String line = logline;
		Pattern m38 = Pattern.compile("^38\\|[^|]*\\|([^|]*)\\|.*");
		Pattern m21 = Pattern.compile("^21\\|[^|]*\\|([^|]*)\\|.*");
		Pattern f38 = Pattern.compile("^38\\|[^|]*\\|([^|]*)");
		Pattern f21 = Pattern.compile("^21\\|[^|]*\\|([^|]*)");
		String s38 = "38|";
		String s21 = "21|";

		// Verify patterns
		Assert.assertTrue(m38.matcher(line).matches());
		Assert.assertFalse(m21.matcher(line).matches());
		Assert.assertTrue(f38.matcher(line).find());
		Assert.assertFalse(f21.matcher(line).find());
		Assert.assertTrue(line.startsWith(s38));
		Assert.assertFalse(line.startsWith(s21));

		// using matches()
		timeIt("matching pattern using matches", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = m38.matcher(line);
				matcher.matches();
				matcher.group(1);
			}
		});
		timeIt("non-matching pattern using matches", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = m21.matcher(line);
				matcher.matches();
			}
		});

		// using find()
		timeIt("matching pattern using find", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = f38.matcher(line);
				matcher.find();
				matcher.group(1);
			}
		});
		timeIt("non-matching pattern using find", () -> {
			for (int i = 0; i < count; i++) {
				Matcher matcher = f21.matcher(line);
				matcher.find();
			}
		});

		// using startsWith()
		timeIt("matching pattern using startsWith", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
				line.split("\\|", 4);
			}
		});
		timeIt("matching pattern using startsWith (increased limit)", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
				line.split("\\|", 12);
			}
		});
		timeIt("matching pattern using startsWith (no limit)", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s38);
				line.split("\\|");
			}
		});
		timeIt("non-matching pattern using startsWith", () -> {
			for (int i = 0; i < count; i++) {
				line.startsWith(s21);
			}
		});

	}

	@Test
	public void testListCopy() {
		List<Integer> collect = IntStream.range(0, 1_000_000)
				.boxed()
				.collect(Collectors.toList());
		timeIt("List Copy", () -> {
			int count = 1000;
			for (int i = 0; i < count; i++) {
				new ArrayList<>(collect);
			}
		});
	}

	@Test
	public void testCompression() throws IOException {
		String inStr = "{\"type\":\"LogLine\",\"line\":[\"38\",\"2021-11-23T20:02:43.3980000-08:00\",\"1081A419\",\"Ruka Kusuo\",\"00444408\",\"3589\",\"3589\",\"461\",\"461\",\"0\",\"0\",\"-108.9822\",\"9.12935\",\"18.00017\",\"2.082842\",\"0\",\"0\",\"0\",\"0\",\"0\",\"0\",\"0\",\"0\",\"0\",\"28F10030\",\"42BAE313\",\"1081A419\",\"01C30031\",\"0\",\"1081A419\",\"32002D\",\"47766C9F\",\"1081A419\",\"\",\"6375258a2fc7832c9a724f6849548da4\"],\"rawLine\":\"38|2021-11-23T20:02:43.3980000-08:00|1081A419|Ruka Kusuo|00444408|3589|3589|461|461|0|0|-108.9822|9.12935|18.00017|2.082842|0|0|0|0|0|0|0|0|0|28F10030|42BAE313|1081A419|01C30031|0|1081A419|32002D|47766C9F|1081A419||6375258a2fc7832c9a724f6849548da4\"}\n";
		byte[] inBytes = inStr.getBytes(StandardCharsets.UTF_8);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		GZIPOutputStream gzip = new GZIPOutputStream(baos);
		gzip.write(inBytes);
		gzip.close();
		byte[] outBytes = baos.toByteArray();
		String outStr = baos.toString(StandardCharsets.UTF_8);

		log.info("Bytes: {} -> {}", inBytes.length, outBytes.length);
		log.info("UTF-8: {} -> {}", inStr.length(), outStr.length());
	}

	@Test
	public void testCompression2() throws IOException {
		String inStrPart1 = "{\"combatants\":[{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275160354,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Wynn Dohz\",\"CurrentHP\":122634,\"MaxHP\":122634,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-109.956116,\"PosY\":4.836665,\"PosZ\":18.7002163,\"Heading\":-1.11362612},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":387,\"BNpcNameID\":387,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Sand Fox\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-108.5957,\"PosY\":3.05665684,\"PosZ\":18.0003319,\"Heading\":2.12961268},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":270451946,\"OwnerID\":0,\"type\":1,\"Job\":36,\"Level\":70,\"Name\":\"Par Hui\",\"CurrentHP\":43848,\"MaxHP\":43848,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.1947,\"PosY\":12.8327637,\"PosZ\":18.0002,\"Heading\":1.11021483},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":273714018,\"OwnerID\":0,\"type\":1,\"Job\":38,\"Level\":80,\"Name\":\"Minecraft Steve\",\"CurrentHP\":124203,\"MaxHP\":124203,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-109.89492,\"PosY\":6.868212,\"PosZ\":18.0001736,\"Heading\":1.18180633},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":141,\"BNpcNameID\":141,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Lesser Panda\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-109.939011,\"PosY\":6.90180874,\"PosZ\":18.87906,\"Heading\":1.18180633},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275524607,\"OwnerID\":0,\"type\":1,\"Job\":30,\"Level\":80,\"Name\":\"Cheese Saganaki\",\"CurrentHP\":138169,\"MaxHP\":138169,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-111.343452,\"PosY\":5.450454,\"PosZ\":18.69995,\"Heading\":-1.137608},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":66,\"BNpcNameID\":66,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Plush Cushion\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-111.343452,\"PosY\":5.450454,\"PosZ\":18.69995,\"Heading\":-1.137608},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275733650,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":80,\"Name\":\"Han Dong\",\"CurrentHP\":135937,\"MaxHP\":135937,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.791733,\"PosY\":3.72841549,\"PosZ\":18.0003338,\"Heading\":1.99655414},{\"CurrentWorldID\":65,\"WorldID\":54,\"WorldName\":\"Faerie\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275504860,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":80,\"Name\":\"Aunna Nadasdy\",\"CurrentHP\":180892,\"MaxHP\":180892,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-104.946342,\"PosY\":3.32162476,\"PosZ\":18.0003338,\"Heading\":-0.726207256},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275512492,\"OwnerID\":0,\"type\":1,\"Job\":38,\"Level\":80,\"Name\":\"Kathein Lulin\",\"CurrentHP\":132888,\"MaxHP\":132888,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-123.735474,\"PosY\":19.8519287,\"PosZ\":18.0,\"Heading\":-0.454496861},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":197,\"BNpcNameID\":197,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Dwarf Rabbit\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-123.31636,\"PosY\":19.5736046,\"PosZ\":18.0,\"Heading\":2.68724251},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275847350,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Mango Kitani\",\"CurrentHP\":122634,\"MaxHP\":122634,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.14212,\"PosY\":8.468278,\"PosZ\":18.050333,\"Heading\":2.029535},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275594397,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":57,\"Name\":\"Vex Eruza\",\"CurrentHP\":12189,\"MaxHP\":12189,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-112.165443,\"PosY\":5.82824373,\"PosZ\":18.69976,\"Heading\":1.99607491},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":247,\"BNpcNameID\":247,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Road Sparrow\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-111.620079,\"PosY\":6.17654562,\"PosZ\":19.0000134,\"Heading\":-0.5170585},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276378295,\"OwnerID\":0,\"type\":1,\"Job\":11,\"Level\":80,\"Name\":\"Tasuki Kozuki\",\"CurrentHP\":3649,\"MaxHP\":3649,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-118.002205,\"PosY\":9.489457,\"PosZ\":18.0,\"Heading\":0.4027238},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":37,\"BNpcNameID\":37,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Fledgling Dodo\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-118.749268,\"PosY\":11.1984291,\"PosZ\":18.0,\"Heading\":-0.7190044},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276891634,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":80,\"Name\":\"Lockon' Stratos\",\"CurrentHP\":194910,\"MaxHP\":194910,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-82.93286,\"PosY\":-12.8328857,\"PosZ\":18.6002789,\"Heading\":0.0236330032},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275050157,\"OwnerID\":0,\"type\":1,\"Job\":31,\"Level\":80,\"Name\":\"Ceres Wisteria\",\"CurrentHP\":131407,\"MaxHP\":131407,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-118.135277,\"PosY\":9.439754,\"PosZ\":18.0,\"Heading\":1.95312285},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":121,\"BNpcNameID\":121,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Doman Magpie\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-116.9146,\"PosY\":10.0389194,\"PosZ\":18.0,\"Heading\":0.1783865},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275386517,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":72,\"Name\":\"Uh'oh Stinky\",\"CurrentHP\":45363,\"MaxHP\":45363,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-89.43317,\"PosY\":1.23590088,\"PosZ\":18.8003311,\"Heading\":1.87047839},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":132,\"BNpcNameID\":132,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Tataru\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-91.11129,\"PosY\":1.1078279,\"PosZ\":18.800333,\"Heading\":1.91886163},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276847043,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Kairhy Phillianor\",\"CurrentHP\":69284,\"MaxHP\":69284,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.035156,\"PosY\":-5.920034,\"PosZ\":18.0003338,\"Heading\":1.55246},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":141,\"BNpcNameID\":141,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Lesser Panda\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-106.924553,\"PosY\":-6.07024336,\"PosZ\":19.3664455,\"Heading\":1.55246},{\"CurrentWorldID\":65,\"WorldID\":54,\"WorldName\":\"Faerie\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276931609,\"OwnerID\":0,\"type\":1,\"Job\":8,\"Level\":68,\"Name\":\"Ruka Kusuo\",\"CurrentHP\":3589,\"MaxHP\":3589,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-97.1542358,\"PosY\":1.96832275,\"PosZ\":18.6003265,\"Heading\":1.58592057},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275998529,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Krul Tsundere\",\"CurrentHP\":116711,\"MaxHP\":116711,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-105.333069,\"PosY\":12.0698242,\"PosZ\":18.0503311,\"Heading\":-2.51849937},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":112,\"BNpcNameID\":112,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Mummy's Little Mummy\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-105.173668,\"PosY\":12.5107355,\"PosZ\":18.0503311,\"Heading\":-0.7244476},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276362797,\"OwnerID\":0,\"type\":1,\"Job\":31,\"Level\":73,\"Name\":\"Machi Komachine\",\"CurrentHP\":58676,\"MaxHP\":58676,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.518059,\"PosY\":15.716466,\"PosZ\":18.6998749,\"Heading\":-2.5286622},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":316,\"BNpcNameID\":316,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Capybara Pup\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-109.516518,\"PosY\":15.2156172,\"PosZ\":18.0,\"Heading\":-2.479335},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275480803,\"OwnerID\":0,\"type\":1,\"Job\":21,\"Level\":62,\"Name\":\"Gyro Chan\",\"CurrentHP\":34032,\"MaxHP\":34032,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.860062,\"PosY\":15.8224716,\"PosZ\":18.6998177,\"Heading\":-2.76576161},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":372,\"BNpcNameID\":372,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Paissa Patissier\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-106.57634,\"PosY\":13.8807592,\"PosZ\":18.0002213,\"Heading\":1.45078468},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275668916,\"OwnerID\":0,\"type\":1,\"Job\":33,\"Level\":80,\"Name\":\"Gundam Mk-Ll\",\"CurrentHP\":127960,\"MaxHP\":127960,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-137.07045,\"PosY\":23.2442913,\"PosZ\":17.9903564,\"Heading\":-2.05045724},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":410,\"BNpcNameID\":410,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Dolphin Calf\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-133.68988,\"PosY\":22.91395,\"PosZ\":18.0,\"Heading\":1.59863269},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276179992,\"OwnerID\":0,\"type\":1,\"Job\":19,\"Level\":80,\"Name\":\"Ichima Kurosaki\",\"CurrentHP\":179065,\"MaxHP\":179065,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-106.606262,\"PosY\":9.06826,\"PosZ\":18.050333,\"Heading\":2.060599},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":255,\"BNpcNameID\":255,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Chimera\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-107.60569,\"PosY\":9.034471,\"PosZ\":18.00033,\"Heading\":1.53700018},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275447884,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Zan Getsu\",\"CurrentHP\":194910,\"MaxHP\":194910,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-104.631165,\"PosY\":12.0087891,\"PosZ\":18.0003319,\"Heading\":-3.09777784},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275494525,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":80,\"Name\":\"Klaus Schweighardt\",\"CurrentHP\":183507,\"MaxHP\":183507,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-124.071106,\"PosY\":11.9172363,\"PosZ\":18.0,\"Heading\":2.84102368},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":260,\"BNpcNameID\":260,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Khloe\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-122.419449,\"PosY\":11.9772787,\"PosZ\":18.0,\"Heading\":-1.600932},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275136485,\"OwnerID\":0,\"type\":1,\"Job\":21,\"Level\":80,\"Name\":\"Mew Too\",\"CurrentHP\":172103,\"MaxHP\":172103,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-95.46177,\"PosY\":-1.70192719,\"PosZ\":18.6003323,\"Heading\":-3.13037539},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":386,\"BNpcNameID\":386,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Allagan Melon\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-95.95203,\"PosY\":-0.8303477,\"PosZ\":18.6003342,\"Heading\":2.62920856},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276001188,\"OwnerID\":0,\"type\":1,\"Job\":36,\"Level\":1,\"Name\":\"Akelv Meeplz\",\"CurrentHP\":146,\"MaxHP\":146,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-98.47367,\"PosY\":-5.73229456,\"PosZ\":18.5165138,\"Heading\":-0.9596636},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":32,\"BNpcNameID\":32,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Buffalo Calf\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-98.9873657,\"PosY\":-4.16142,\"PosZ\":18.5112514,\"Heading\":-2.97945619},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275037533,\"OwnerID\":0,\"type\":1,\"Job\":31,\"Level\":80,\"Name\":\"Silverjuin Axeman\",\"CurrentHP\":131496,\"MaxHP\":131496,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-82.78027,\"PosY\":11.2763672,\"PosZ\":18.60033,\"Heading\":3.13852429},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275412724,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":80,\"Name\":\"Arcia Sonata\",\"CurrentHP\":180987,\"MaxHP\":180987,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-94.8538742,\"PosY\":-7.024427,\"PosZ\":18.6003323,\"Heading\":-1.99684215},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1013,\"BNpcNameID\":5478,\"PartyType\":0,\"ID\":1075805567,\"OwnerID\":274979492,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Ruby Carbuncle\",\"CurrentHP\":3423,\"MaxHP\":3423,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-98.27491,\"PosY\":0.561298132,\"PosZ\":18.60033,\"Heading\":0.4952433},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275906548,\"OwnerID\":0,\"type\":1,\"Job\":28,\"Level\":80,\"Name\":\"Myth Nepitii\",\"CurrentHP\":115318,\"MaxHP\":115318,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-97.24357,\"PosY\":5.6242466,\"PosZ\":18.60033,\"Heading\":-1.253713},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":255,\"BNpcNameID\":255,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Chimera\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-96.26482,\"PosY\":5.82929754,\"PosZ\":18.6003323,\"Heading\":-1.77731192},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276356544,\"OwnerID\":0,\"type\":1,\"Job\":31,\"Level\":80,\"Name\":\"Lana Crim\",\"CurrentHP\":127562,\"MaxHP\":127562,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-100.188271,\"PosY\":6.10902262,\"PosZ\":18.2504234,\"Heading\":-1.96088886},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":341,\"BNpcNameID\":341,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Eden Minor\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-99.19717,\"PosY\":5.97591257,\"PosZ\":18.4006386,\"Heading\":-1.43729043},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275630180,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":80,\"Name\":\"Faust Angura\",\"CurrentHP\":130014,\"MaxHP\":130014,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-106.037712,\"PosY\":15.0761557,\"PosZ\":18.7001419,\"Heading\":-2.80957675},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":358,\"BNpcNameID\":358,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Cute Justice\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-106.10717,\"PosY\":15.4342642,\"PosZ\":18.70013,\"Heading\":2.9500134},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275914593,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Moose Dorne\",\"CurrentHP\":100600,\"MaxHP\":100600,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-99.59564,\"PosY\":23.3004761,\"PosZ\":18.8826942,\"Heading\":-1.76175606},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":274979492,\"OwnerID\":0,\"type\":1,\"Job\":27,\"Level\":50,\"Name\":\"Markie Kunakato\",\"CurrentHP\":3498,\"MaxHP\":3498,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-98.33209,\"PosY\":2.73054457,\"PosZ\":18.6003265,\"Heading\":0.7589965},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":395,\"BNpcNameID\":395,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"2P Automaton\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-98.56532,\"PosY\":1.75812292,\"PosZ\":18.6003265,\"Heading\":0.235398278},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275327484,\"OwnerID\":0,\"type\":1,\"Job\":28,\"Level\":80,\"Name\":\"Shirley Mairze\",\"CurrentHP\":125087,\"MaxHP\":125087,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-98.0221558,\"PosY\":8.15043,\"PosZ\":18.4103,\"Heading\":-1.54133868},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":350,\"BNpcNameID\":350,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Butterfly Effect\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-97.14178,\"PosY\":8.624705,\"PosZ\":18.4817619,\"Heading\":-2.06493616},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275038647,\"OwnerID\":0,\"type\":1,\"Job\":23,\"Level\":68,\"Name\":\"Baby Dollz\",\"CurrentHP\":26527,\"MaxHP\":26527,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-90.62335,\"PosY\":15.0606079,\"PosZ\":18.0003338,\"Heading\":-2.28465962},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276179314,\"OwnerID\":0,\"type\":1,\"Job\":20,\"Level\":80,\"Name\":\"Keaira Chanel\",\"CurrentHP\":127738,\"MaxHP\":127738,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-92.63757,\"PosY\":5.081238,\"PosZ\":18.7571049,\"Heading\":1.8272388},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":394,\"BNpcNameID\":394,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"2B Automaton\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-94.2964554,\"PosY\":4.785055,\"PosZ\":18.6003361,\"Heading\":1.80983424},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276412206,\"OwnerID\":0,\"type\":1,\"Job\":31,\"Level\":80,\"Name\":\"Demarcus Askickerman\",\"CurrentHP\":117396,\"MaxHP\":117396,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-103.136154,\"PosY\":9.7996645,\"PosZ\":18.0003338,\"Heading\":2.18916774},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":145,\"BNpcNameID\":145,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Iceheart\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-104.131668,\"PosY\":9.894295,\"PosZ\":18.0003319,\"Heading\":1.66556823},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":268854691,\"OwnerID\":0,\"type\":1,\"Job\":30,\"Level\":80,\"Name\":\"Kokona Watts\",\"CurrentHP\":124710,\"MaxHP\":124710,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-102.644768,\"PosY\":15.9154215,\"PosZ\":18.7003326,\"Heading\":2.32780337},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":274348230,\"OwnerID\":0,\"type\":1,\"Job\":25,\"Level\":80,\"Name\":\"Dietrich Faust\",\"CurrentHP\":123982,\"MaxHP\":123982,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-97.69205,\"PosY\":9.183209,\"PosZ\":18.3648129,\"Heading\":-2.81580853},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":142,\"BNpcNameID\":142,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Page 63\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-95.91282,\"PosY\":10.8624964,\"PosZ\":18.4276886,\"Heading\":1.63567293},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275204922,\"OwnerID\":0,\"type\":1,\"Job\":31,\"Level\":40,\"Name\":\"Gineus Frost\",\"CurrentHP\":1404,\"MaxHP\":1404,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-101.9962,\"PosY\":16.30539,\"PosZ\":18.5003338,\"Heading\":2.357237},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275098063,\"OwnerID\":0,\"type\":1,\"Job\":21,\"Level\":42,\"Name\":\"Mister Breakfast\",\"CurrentHP\":2631,\"MaxHP\":2631,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-103.748,\"PosY\":13.6998644,\"PosZ\":19.9459457,\"Heading\":2.10776925},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":37,\"BNpcNameID\":37,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Fledgling Dodo\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-104.019875,\"PosY\":14.7225189,\"PosZ\":19.5229874,\"Heading\":2.20180941},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275903003,\"OwnerID\":0,\"type\":1,\"Job\":30,\"Level\":79,\"Name\":\"Trintas Targaryen\",\"CurrentHP\":73016,\"MaxHP\":73016,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-122.762268,\"PosY\":12.6457272,\"PosZ\":18.0,\"Heading\":-2.526457},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":130,\"BNpcNameID\":130,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Alphinaud\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-122.67086,\"PosY\":13.6415405,\"PosZ\":18.0,\"Heading\":-3.050058},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276785752,\"OwnerID\":0,\"type\":1,\"Job\":21,\"Level\":80,\"Name\":\"Fae Aluminia\",\"CurrentHP\":184199,\"MaxHP\":184199,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-87.11377,\"PosY\":11.8562012,\"PosZ\":18.6003323,\"Heading\":3.13852429},{\"CurrentWorldID\":65,\"WorldID\":57,\"WorldName\":\"Siren\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":268617232,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":70,\"Name\":\"Elana Astaria\",\"CurrentHP\":34848,\"MaxHP\":34848,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-86.7780762,\"PosY\":3.00598145,\"PosZ\":19.3376675,\"Heading\":2.40987277},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":274396669,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Blaze Tenebris\",\"CurrentHP\":185680,\"MaxHP\":185680,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-81.22388,\"PosY\":-12.6192627,\"PosZ\":18.6002865,\"Heading\":0.324969053},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":272135597,\"OwnerID\":0,\"type\":1,\"Job\":22,\"Level\":52,\"Name\":\"Ocho Cho\",\"CurrentHP\":5420,\"MaxHP\":5420,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-96.81854,\"PosY\":1.38848877,\"PosZ\":18.60033,\"Heading\":1.55696607},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276021787,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":80,\"Name\":\"Kyoji Rokos\",\"CurrentHP\":131185,\"MaxHP\":131185,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-93.6141357,\"PosY\":19.57727,\"PosZ\":18.0003338,\"Heading\":1.27931142},{\"CurrentWorldID\":65,\"WorldID\":40,\"WorldName\":\"Jenova\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":268730499,\"OwnerID\":0,\"type\":1,\"Job\":17,\"Level\":80,\"Name\":\"Hazel Irisveil\",\"CurrentHP\":23383,\"MaxHP\":23383,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-121.112022,\"PosY\":20.0377216,\"PosZ\":18.0,\"Heading\":-0.7110591},{\"CurrentWorldID\":65,\"WorldID\":57,\"WorldName\":\"Siren\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275515383,\"OwnerID\":0,\"type\":1,\"Job\":4,\"Level\":22,\"Name\":\"Theta Ant\",\"CurrentHP\":447,\"MaxHP\":447,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-93.21747,\"PosY\":0.8086548,\"PosZ\":18.8003311,\"Heading\":-2.50172138},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":272859567,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":80,\"Name\":\"Dumb Cat\",\"CurrentHP\":118500,\"MaxHP\":118500,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-93.3699951,\"PosY\":2.79229736,\"PosZ\":18.8003311,\"Heading\":1.46607661},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275303870,\"OwnerID\":0,\"type\":1,\"Job\":15,\"Level\":80,\"Name\":\"Ariae Hon\",\"CurrentHP\":3627,\"MaxHP\":3627,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-119.124275,\"PosY\":19.37121,\"PosZ\":18.0,\"Heading\":0.4237206},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275753659,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":64,\"Name\":\"Lancelot Whiteknight\",\"CurrentHP\":35179,\"MaxHP\":35179,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-121.385559,\"PosY\":19.21106,\"PosZ\":18.0,\"Heading\":0.371851921},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":386,\"BNpcNameID\":386,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Allagan Melon\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-119.984764,\"PosY\":18.27818,\"PosZ\":18.0,\"Heading\":-0.9832646},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276950417,\"OwnerID\":0,\"type\":1,\"Job\":12,\"Level\":80,\"Name\":\"Mio Fox\",\"CurrentHP\":4775,\"MaxHP\":4775,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-125.871765,\"PosY\":22.9342651,\"PosZ\":18.2,\"Heading\":2.21352029},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":221,\"BNpcNameID\":221,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Rikku\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-127.231041,\"PosY\":23.9266663,\"PosZ\":18.1999989,\"Heading\":2.20144367},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275971014,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Duncan Maccloud\",\"CurrentHP\":174939,\"MaxHP\":174939,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-127.115494,\"PosY\":18.86677,\"PosZ\":18.0,\"Heading\":2.40392852},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":409,\"BNpcNameID\":409,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Anteater\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-128.417084,\"PosY\":19.3719139,\"PosZ\":17.9999981,\"Heading\":-1.00559413},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275968930,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Death Snail\",\"CurrentHP\":194815,\"MaxHP\":194815,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-84.75152,\"PosY\":-11.1238594,\"PosZ\":18.60032,\"Heading\":0.0236330032},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":272554992,\"OwnerID\":0,\"type\":1,\"Job\":22,\"Level\":80,\"Name\":\"Golarion Godhand\",\"CurrentHP\":121659,\"MaxHP\":121659,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-106.3621,\"PosY\":-9.610778,\"PosZ\":18.5003262,\"Heading\":1.18410707},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276671266,\"OwnerID\":0,\"type\":1,\"Job\":33,\"Level\":80,\"Name\":\"Austere Dune\",\"CurrentHP\":102567,\"MaxHP\":102567,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-102.805977,\"PosY\":11.9204159,\"PosZ\":17.9973869,\"Heading\":-1.93874168},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276272490,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":78,\"Name\":\"Yoon Hugehuge\",\"CurrentHP\":64626,\"MaxHP\":64626,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-136.750381,\"PosY\":23.1995277,\"PosZ\":18.0,\"Heading\":2.13068366},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275211988,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":44,\"Name\":\"David Mustaine\",\"CurrentHP\":2832,\"MaxHP\":2832,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-90.176384,\"PosY\":-7.4306283,\"PosZ\":18.8003254,\"Heading\":-2.64227438},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":284,\"BNpcNameID\":284,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Byakko Cub\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-92.04556,\"PosY\":-8.098558,\"PosZ\":18.6003361,\"Heading\":-2.75482726},{\"CurrentWorldID\":65,\"WorldID\":73,\"WorldName\":\"Adamantoise\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275205025,\"OwnerID\":0,\"type\":1,\"Job\":13,\"Level\":80,\"Name\":\"Khaose Surestorm\",\"CurrentHP\":3649,\"MaxHP\":3649,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-121.721252,\"PosY\":11.24585,\"PosZ\":18.0,\"Heading\":-2.49290085},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":369,\"BNpcNameID\":369,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"The Behelmeted Serpent Of Ronka\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-120.196815,\"PosY\":10.5326939,\"PosZ\":17.9999981,\"Heading\":-1.56330776},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275421760,\"OwnerID\":0,\"type\":1,\"Job\":22,\"Level\":80,\"Name\":\"Ray Leonhart\",\"CurrentHP\":132090,\"MaxHP\":132090,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-92.94275,\"PosY\":2.5177002,\"PosZ\":18.800333,\"Heading\":0.162940025},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276710854,\"OwnerID\":0,\"type\":1,\"Job\":6,\"Level\":31,\"Name\":\"Fishydip Uwu\",\"CurrentHP\":815,\"MaxHP\":815,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-132.437134,\"PosY\":18.4489384,\"PosZ\":17.9903564,\"Heading\":-1.13041735},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":19,\"BNpcNameID\":19,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Coeurl Kitten\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-122.983681,\"PosY\":13.4386854,\"PosZ\":18.0,\"Heading\":-1.14751112},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276301027,\"OwnerID\":0,\"type\":1,\"Job\":25,\"Level\":80,\"Name\":\"Noel Masters\",\"CurrentHP\":128402,\"MaxHP\":128402,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-119.810783,\"PosY\":19.290741,\"PosZ\":18.0,\"Heading\":0.3487463},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":384,\"BNpcNameID\":384,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Weatherproof Gaelicat\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-120.66671,\"PosY\":19.32539,\"PosZ\":18.0,\"Heading\":-2.973265},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275220338,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Devii Donuts\",\"CurrentHP\":126545,\"MaxHP\":126545,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-105.699341,\"PosY\":11.5509644,\"PosZ\":18.0503311,\"Heading\":-0.8087559},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":255,\"BNpcNameID\":255,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Chimera\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-104.950111,\"PosY\":10.2173147,\"PosZ\":18.0003319,\"Heading\":-0.478709757},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275110571,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":56,\"Name\":\"Hatsumi Kana\",\"CurrentHP\":8133,\"MaxHP\":8133,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-87.36085,\"PosY\":13.5565386,\"PosZ\":18.60033,\"Heading\":1.88112044},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":231,\"BNpcNameID\":231,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Anima\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-88.84118,\"PosY\":12.0087032,\"PosZ\":18.6003246,\"Heading\":3.08805871},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":277139892,\"OwnerID\":0,\"type\":1,\"Job\":23,\"Level\":80,\"Name\":\"Ash Shalash\",\"CurrentHP\":119849,\"MaxHP\":119849,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-84.61133,\"PosY\":13.534729,\"PosZ\":18.60033,\"Heading\":3.13852429},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1013,\"BNpcNameID\":1402,\"PartyType\":0,\"ID\":1075807839,\"OwnerID\":276668037,\"type\":2,\"Job\":0,\"Level\":80,\"Name\":\"Ifrit-Egi\",\"CurrentHP\":68998,\"MaxHP\":68998,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-123.391441,\"PosY\":18.08647,\"PosZ\":18.0,\"Heading\":-0.219314575},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276408300,\"OwnerID\":0,\"type\":1,\"Job\":21,\"Level\":50,\"Name\":\"Lina Hochratova\",\"CurrentHP\":3952,\"MaxHP\":3952,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-135.759521,\"PosY\":15.3047485,\"PosZ\":18.2,\"Heading\":-2.881579},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":41,\"BNpcNameID\":41,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Goobbue Sproutling\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-137.315323,\"PosY\":16.4869175,\"PosZ\":18.1999989,\"Heading\":-1.80273688},{\"CurrentWorldID\":65,\"WorldID\":63,\"WorldName\":\"Gilgamesh\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275043323,\"OwnerID\":0,\"type\":1,\"Job\":27,\"Level\":80,\"Name\":\"Ai Ai\",\"CurrentHP\":122611,\"MaxHP\":122611,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-91.2337646,\"PosY\":15.3657837,\"PosZ\":18.0003338,\"Heading\":2.56758761},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":273,\"BNpcNameID\":273,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Axolotl Eft\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-92.4443,\"PosY\":13.7229815,\"PosZ\":18.0440445,\"Heading\":-3.09655428},{\"CurrentWorldID\":65,\"WorldID\":63,\"WorldName\":\"Gilgamesh\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276629707,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":80,\"Name\":\"King Vamp\",\"CurrentHP\":135981,\"MaxHP\":135981,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-91.02008,\"PosY\":14.7554321,\"PosZ\":18.0003338,\"Heading\":1.24738479},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":267,\"BNpcNameID\":267,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Mameshiba\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-92.99207,\"PosY\":16.5625,\"PosZ\":18.0003338,\"Heading\":-1.37584758},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275219032,\"OwnerID\":0,\"type\":1,\"Job\":22,\"Level\":80,\"Name\":\"Enzo Rosso\",\"CurrentHP\":136665,\"MaxHP\":136665,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-105.984428,\"PosY\":-5.23338032,\"PosZ\":17.9903564,\"Heading\":-0.749545},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":333,\"BNpcNameID\":333,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Black Hayate\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-103.236351,\"PosY\":-7.93076944,\"PosZ\":18.0003338,\"Heading\":-0.8774657},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275613148,\"OwnerID\":0,\"type\":1,\"Job\":33,\"Level\":80,\"Name\":\"Tez Celah\",\"CurrentHP\":121750,\"MaxHP\":121750,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-94.24298,\"PosY\":22.9447632,\"PosZ\":19.1032982,\"Heading\":3.03171945},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":138,\"BNpcNameID\":138,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Ugly Duckling\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-93.2780151,\"PosY\":23.7617588,\"PosZ\":18.9831123,\"Heading\":-0.4583235},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":273774275,\"OwnerID\":0,\"type\":1,\"Job\":25,\"Level\":71,\"Name\":\"Dioress Pradass\",\"CurrentHP\":50844,\"MaxHP\":50844,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-121.660278,\"PosY\":11.3984375,\"PosZ\":18.0,\"Heading\":-2.12013769},";
		String inStrPart2 = "{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":16,\"BNpcNameID\":16,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Bluebird\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-121.653534,\"PosY\":11.328618,\"PosZ\":19.3304615,\"Heading\":-2.12013769},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275280427,\"OwnerID\":0,\"type\":1,\"Job\":27,\"Level\":70,\"Name\":\"Lennethelia Brayv\",\"CurrentHP\":33115,\"MaxHP\":33115,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-115.953369,\"PosY\":13.2905273,\"PosZ\":18.0,\"Heading\":-1.77680838},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":272,\"BNpcNameID\":272,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Salt & Pepper Seal\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-116.464806,\"PosY\":12.4924564,\"PosZ\":18.0,\"Heading\":2.71762824},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276210504,\"OwnerID\":0,\"type\":1,\"Job\":19,\"Level\":80,\"Name\":\"Kage Senshi\",\"CurrentHP\":186058,\"MaxHP\":186058,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-88.1967239,\"PosY\":17.75139,\"PosZ\":18.00033,\"Heading\":2.3098743},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":395,\"BNpcNameID\":395,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"2P Automaton\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-89.1736,\"PosY\":17.9652061,\"PosZ\":18.00033,\"Heading\":1.78627539},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275736150,\"OwnerID\":0,\"type\":1,\"Job\":8,\"Level\":39,\"Name\":\"Ghost Heals\",\"CurrentHP\":1017,\"MaxHP\":1017,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-97.80306,\"PosY\":-1.010669,\"PosZ\":18.60033,\"Heading\":1.55696607},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":70,\"BNpcNameID\":70,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Minion Of Light\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-98.66209,\"PosY\":-1.522598,\"PosZ\":18.6003323,\"Heading\":1.03336549},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275137861,\"OwnerID\":0,\"type\":1,\"Job\":23,\"Level\":80,\"Name\":\"Slate Lord\",\"CurrentHP\":63715,\"MaxHP\":63715,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-82.90228,\"PosY\":-13.4737549,\"PosZ\":18.6002617,\"Heading\":0.0236330032},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":389,\"BNpcNameID\":389,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Dáinsleif F1\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-82.40813,\"PosY\":-14.338459,\"PosZ\":18.6002522,\"Heading\":-0.499963582},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276668037,\"OwnerID\":0,\"type\":1,\"Job\":27,\"Level\":80,\"Name\":\"Selysa Silverbloom\",\"CurrentHP\":69174,\"MaxHP\":69174,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-124.28479,\"PosY\":20.0656128,\"PosZ\":18.0,\"Heading\":0.367058277},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275345430,\"OwnerID\":0,\"type\":1,\"Job\":34,\"Level\":80,\"Name\":\"Fox Style\",\"CurrentHP\":120047,\"MaxHP\":120047,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-92.82062,\"PosY\":3.12805176,\"PosZ\":18.800333,\"Heading\":-2.919354},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":274884857,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":80,\"Name\":\"Tiv Auxilium\",\"CurrentHP\":193429,\"MaxHP\":193429,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-91.93567,\"PosY\":27.7255859,\"PosZ\":19.08296,\"Heading\":0.157091618},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":279,\"BNpcNameID\":279,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"White Whittret\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-92.08426,\"PosY\":27.3072815,\"PosZ\":19.1859951,\"Heading\":0.157091618},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276450722,\"OwnerID\":0,\"type\":1,\"Job\":11,\"Level\":45,\"Name\":\"Ezner En'zeri\",\"CurrentHP\":1220,\"MaxHP\":1220,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-118.822021,\"PosY\":13.534729,\"PosZ\":18.0,\"Heading\":-1.99089789},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":258,\"BNpcNameID\":258,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Ghido\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-120.691956,\"PosY\":13.8342562,\"PosZ\":18.0,\"Heading\":-1.06823552},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275386647,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":69,\"Name\":\"Supay Q'\",\"CurrentHP\":35765,\"MaxHP\":35765,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-124.620483,\"PosY\":20.4012451,\"PosZ\":18.0,\"Heading\":0.2162466},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":268854651,\"OwnerID\":0,\"type\":1,\"Job\":27,\"Level\":57,\"Name\":\"Rikako Kano\",\"CurrentHP\":6321,\"MaxHP\":6321,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-119.768066,\"PosY\":19.4552,\"PosZ\":18.0,\"Heading\":-0.114810944},{\"CurrentWorldID\":65,\"WorldID\":54,\"WorldName\":\"Faerie\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":274892576,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Gin Fuyuki\",\"CurrentHP\":194091,\"MaxHP\":194091,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-118.303223,\"PosY\":19.2720947,\"PosZ\":18.0,\"Heading\":-0.277415276},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":387,\"BNpcNameID\":387,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Sand Fox\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-119.3793,\"PosY\":18.0601749,\"PosZ\":18.0,\"Heading\":2.9527545},{\"CurrentWorldID\":65,\"WorldID\":79,\"WorldName\":\"Cactuar\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276069201,\"OwnerID\":0,\"type\":1,\"Job\":28,\"Level\":80,\"Name\":\"Audrey Bertrand\",\"CurrentHP\":83959,\"MaxHP\":83959,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-84.36719,\"PosY\":-11.0018311,\"PosZ\":18.6003246,\"Heading\":0.0236330032},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":385,\"BNpcNameID\":385,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Ephemeral Necromancer\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-83.86012,\"PosY\":-11.8634586,\"PosZ\":18.6003036,\"Heading\":-0.499963582},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276488329,\"OwnerID\":0,\"type\":1,\"Job\":19,\"Level\":80,\"Name\":\"Cloudy De'storm\",\"CurrentHP\":191476,\"MaxHP\":191476,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-107.08622,\"PosY\":14.0364256,\"PosZ\":18.0001564,\"Heading\":1.18784642},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1013,\"BNpcNameID\":1402,\"PartyType\":0,\"ID\":1075807536,\"OwnerID\":275280427,\"type\":2,\"Job\":0,\"Level\":70,\"Name\":\"Ifrit-Egi\",\"CurrentHP\":31186,\"MaxHP\":31186,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-117.143555,\"PosY\":15.1216431,\"PosZ\":18.0,\"Heading\":2.02176976},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275503387,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Ren Narana\",\"CurrentHP\":187570,\"MaxHP\":187570,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-91.99664,\"PosY\":-1.48016357,\"PosZ\":18.800333,\"Heading\":2.76020074},{\"CurrentWorldID\":65,\"WorldID\":99,\"WorldName\":\"Sargatanas\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":277140216,\"OwnerID\":0,\"type\":1,\"Job\":35,\"Level\":80,\"Name\":\"Kinsenya Liska\",\"CurrentHP\":122236,\"MaxHP\":122236,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-119.096741,\"PosY\":19.6383667,\"PosZ\":18.0,\"Heading\":-0.051820755},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275724475,\"OwnerID\":0,\"type\":1,\"Job\":20,\"Level\":80,\"Name\":\"Aililli Nightingale\",\"CurrentHP\":127716,\"MaxHP\":127716,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-91.4779053,\"PosY\":-2.456726,\"PosZ\":18.8003311,\"Heading\":-0.51183033},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":350,\"BNpcNameID\":350,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Butterfly Effect\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-92.2006454,\"PosY\":-1.61069071,\"PosZ\":18.8003311,\"Heading\":-0.156843081},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275821009,\"OwnerID\":0,\"type\":1,\"Job\":36,\"Level\":70,\"Name\":\"Ajitte Solainteau\",\"CurrentHP\":43418,\"MaxHP\":43418,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-83.64078,\"PosY\":13.5026693,\"PosZ\":18.60033,\"Heading\":3.13852429},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276666213,\"OwnerID\":0,\"type\":1,\"Job\":22,\"Level\":61,\"Name\":\"Vincent Brucemont\",\"CurrentHP\":7987,\"MaxHP\":7987,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-98.22235,\"PosY\":2.76184082,\"PosZ\":18.6003265,\"Heading\":1.68294644},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":115,\"BNpcNameID\":115,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Assassin Fry\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-99.7681351,\"PosY\":1.58479738,\"PosZ\":18.4659672,\"Heading\":-2.51817918},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":277146380,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":66,\"Name\":\"Yuno Arai\",\"CurrentHP\":25172,\"MaxHP\":25172,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-78.29407,\"PosY\":-1.02233887,\"PosZ\":18.8003254,\"Heading\":2.94974637},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275327054,\"OwnerID\":0,\"type\":1,\"Job\":37,\"Level\":80,\"Name\":\"Phaunna Kyo\",\"CurrentHP\":198784,\"MaxHP\":198784,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-90.3486938,\"PosY\":14.9995728,\"PosZ\":18.0003319,\"Heading\":-1.2926383},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":276923164,\"OwnerID\":0,\"type\":1,\"Job\":32,\"Level\":54,\"Name\":\"Enyo Cattegiris\",\"CurrentHP\":10411,\"MaxHP\":10411,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-82.65814,\"PosY\":12.0087891,\"PosZ\":18.6003284,\"Heading\":3.13852429},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":101,\"BNpcNameID\":101,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Puff Of Darkness\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-83.15535,\"PosY\":12.8842049,\"PosZ\":19.7171345,\"Heading\":2.61502218},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275797733,\"OwnerID\":0,\"type\":1,\"Job\":19,\"Level\":70,\"Name\":\"Kallen Stark\",\"CurrentHP\":74281,\"MaxHP\":74281,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-134.416748,\"PosY\":17.04425,\"PosZ\":18.0017834,\"Heading\":1.83596349},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275542441,\"OwnerID\":0,\"type\":1,\"Job\":16,\"Level\":80,\"Name\":\"Maleficanum Azuretina\",\"CurrentHP\":23428,\"MaxHP\":23428,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-92.63757,\"PosY\":1.41906738,\"PosZ\":18.800333,\"Heading\":-2.51303458},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275482870,\"OwnerID\":0,\"type\":1,\"Job\":19,\"Level\":80,\"Name\":\"Vampire Lord\",\"CurrentHP\":203100,\"MaxHP\":203100,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-106.2486,\"PosY\":12.1003418,\"PosZ\":18.0003338,\"Heading\":2.386575},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":26,\"BNpcNameID\":26,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Baby Bat\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-107.791794,\"PosY\":12.2559452,\"PosZ\":18.00016,\"Heading\":-1.39716852},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":277148685,\"OwnerID\":0,\"type\":1,\"Job\":4,\"Level\":6,\"Name\":\"Yuu Demonshadow\",\"CurrentHP\":160,\"MaxHP\":160,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-80.06415,\"PosY\":-0.5340576,\"PosZ\":19.3102856,\"Heading\":-1.45553041},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":52,\"BNpcNameID\":52,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Wind-Up Airship\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-78.54352,\"PosY\":-1.67586684,\"PosZ\":18.8003254,\"Heading\":-0.723366261},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275462953,\"OwnerID\":0,\"type\":1,\"Job\":27,\"Level\":80,\"Name\":\"Klonedrone Surama\",\"CurrentHP\":112821,\"MaxHP\":112821,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-122.8504,\"PosY\":16.80011,\"PosZ\":18.0,\"Heading\":1.96414828},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":380,\"BNpcNameID\":380,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":9,\"Job\":0,\"Level\":0,\"Name\":\"Lalinator 5.H0\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-124.036636,\"PosY\":18.0935841,\"PosZ\":18.0,\"Heading\":2.41813231},{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":274611192,\"OwnerID\":0,\"type\":1,\"Job\":28,\"Level\":80,\"Name\":\"Ana Skudy\",\"CurrentHP\":102014,\"MaxHP\":102014,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":-87.14429,\"PosY\":-6.85137939,\"PosZ\":18.8003254,\"Heading\":1.2629168},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1033775,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Rowena's Representative\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-107.438843,\"PosY\":-4.623535,\"PosZ\":18.0003319,\"Heading\":1.2377975},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1009434,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Itinerant Moogle\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-90.89807,\"PosY\":21.5914917,\"PosZ\":18.0003338,\"Heading\":2.82577968},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001202,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Sorcha\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-136.186829,\"PosY\":12.2835083,\"PosZ\":18.2,\"Heading\":0.897330761},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001204,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Engerrand\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-129.533875,\"PosY\":28.6105957,\"PosZ\":18.1999989,\"Heading\":-2.375837},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1009716,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Skywatcher\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-82.8718262,\"PosY\":23.0563354,\"PosZ\":18.0003242,\"Heading\":0.890633345},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001217,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Swozblaet\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-140.856018,\"PosY\":17.0137329,\"PosZ\":18.1999989,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003275,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Frydwyb\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-147.1123,\"PosY\":14.3587036,\"PosZ\":18.2,\"Heading\":0.327557564},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003252,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Blue Lily\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-139.574341,\"PosY\":31.9676514,\"PosZ\":18.2,\"Heading\":-2.02485},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001023,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Sundhimal\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-78.62976,\"PosY\":-22.62915,\"PosZ\":18.0003338,\"Heading\":-0.22688891},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001787,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Bango Zango\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-62.1195679,\"PosY\":9.414734,\"PosZ\":18.0003338,\"Heading\":-2.09875751},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003355,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"J'nasshym\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-60.8682861,\"PosY\":7.4921875,\"PosZ\":18.0003338,\"Heading\":-1.8527416},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001203,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Iron Thunder\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-156.6034,\"PosY\":20.9200439,\"PosZ\":18.2,\"Heading\":0.640398741},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003253,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Gerulf\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-149.950439,\"PosY\":36.9420166,\"PosZ\":18.2,\"Heading\":-2.126082},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003631,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-107.71344,\"PosY\":4.196167,\"PosZ\":17.9903564,\"Heading\":1.88553071},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1005412,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Troubled Adventurer\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-108.140747,\"PosY\":-0.228942871,\"PosZ\":17.9903564,\"Heading\":1.54766631},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003274,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Glazrael\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-110.460083,\"PosY\":15.7014771,\"PosZ\":18.0,\"Heading\":2.86048579},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001209,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Sweetnix Rosycheeks\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-103.746094,\"PosY\":16.03717,\"PosZ\":18.5003338,\"Heading\":-0.538867},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1011004,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Y'shtola\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-97.2762451,\"PosY\":19.4857788,\"PosZ\":17.9903564,\"Heading\":2.33911633},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003277,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Rose\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-136.67511,\"PosY\":16.4949951,\"PosZ\":18.2,\"Heading\":3.0513742},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1005371,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-141.632034,\"PosY\":30.16863,\"PosZ\":18.2,\"Heading\":0.9650629},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1009487,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-77.37854,\"PosY\":-21.3168945,\"PosZ\":17.9903564,\"Heading\":-2.48062873},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1029082,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Sophie\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-152.758118,\"PosY\":22.3239136,\"PosZ\":18.1734,\"Heading\":-1.92579854},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1029081,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Radovan\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-153.8263,\"PosY\":22.8427124,\"PosZ\":18.1734,\"Heading\":-2.18379879},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1026928,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Zealous Yellowjacket\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-64.34735,\"PosY\":-11.0322876,\"PosZ\":17.9903564,\"Heading\":-1.0660851},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003604,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Ahldskyf\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-60.44104,\"PosY\":-4.348877,\"PosZ\":18.00033,\"Heading\":0.208576441},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1010544,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-94.31604,\"PosY\":-45.5177,\"PosZ\":19.0584717,\"Heading\":0.488244534},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001210,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Mamool Ja Marketeer\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-153.795715,\"PosY\":35.324585,\"PosZ\":18.2,\"Heading\":1.160618},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1028721,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Eager Lominsan\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-70.54254,\"PosY\":42.1606445,\"PosZ\":19.97406,\"Heading\":0.06764006},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1028722,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Enthusiastic Eavesdropper\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-70.26782,\"PosY\":43.62549,\"PosZ\":19.97406,\"Heading\":-3.074576},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1028798,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Nashmeira\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-70.38995,\"PosY\":45.3344727,\"PosZ\":19.97406,\"Heading\":1.67335868},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1028799,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Ranaa Mihgo\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-68.89453,\"PosY\":45.3955078,\"PosZ\":19.97406,\"Heading\":-1.78380728},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1022168,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Hilda\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-60.7157,\"PosY\":42.3742676,\"PosZ\":19.97406,\"Heading\":1.32379758},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1022169,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Rostnsthal\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-58.8540649,\"PosY\":42.83203,\"PosZ\":19.97406,\"Heading\":-1.90643167},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003596,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Ryssfloh\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-41.4282837,\"PosY\":-5.661133,\"PosZ\":19.9999847,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1009153,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Deputy Postmoogle\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-39.1089478,\"PosY\":5.416931,\"PosZ\":19.97406,\"Heading\":3.07668471},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003352,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-115.220886,\"PosY\":83.05481,\"PosZ\":21.8044624,\"Heading\":0.623045146},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003351,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-116.19751,\"PosY\":83.14636,\"PosZ\":21.86959,\"Heading\":0.101963043},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1010419,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Anselm\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-117.845459,\"PosY\":83.604126,\"PosZ\":21.3473511,\"Heading\":0.741451263},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003354,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-111.5282,\"PosY\":84.67224,\"PosZ\":19.8496151,\"Heading\":-0.8006065},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003350,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-113.298279,\"PosY\":85.9845,\"PosZ\":19.8333054,\"Heading\":-0.8853603},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003353,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-115.861816,\"PosY\":87.02222,\"PosZ\":19.8422947,\"Heading\":1.64958167},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003501,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-192.950378,\"PosY\":32.3947754,\"PosZ\":16.0,\"Heading\":-2.27555156},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1010422,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Anselm\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-22.5375977,\"PosY\":3.86047363,\"PosZ\":19.97406,\"Heading\":1.76952147},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003520,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-170.122925,\"PosY\":68.7113,\"PosZ\":12.0,\"Heading\":1.46320033},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1010423,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Fiebras\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-20.73706,\"PosY\":4.257263,\"PosZ\":19.97406,\"Heading\":1.97210622},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003521,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-170.702759,\"PosY\":70.4508057,\"PosZ\":12.0,\"Heading\":-0.86235},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003529,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-171.252075,\"PosY\":71.4885254,\"PosZ\":12.0,\"Heading\":2.82031465},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1001038,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-184.802063,\"PosY\":56.381958,\"PosZ\":15.999999,\"Heading\":-2.02282429},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1021476,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Arya\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-196.1242,\"PosY\":37.4608154,\"PosZ\":15.487854,\"Heading\":-0.447402269},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003505,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Dodozan\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-186.205872,\"PosY\":56.9313965,\"PosZ\":16.0,\"Heading\":1.76357722},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1021475,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"X'rhun Tia\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-196.948242,\"PosY\":37.5828857,\"PosZ\":15.5184326,\"Heading\":0.156995773},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1021477,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Riol\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-196.734619,\"PosY\":38.92566,\"PosZ\":15.487854,\"Heading\":-2.8744843},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1014946,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"Quimperain\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-189.89856,\"PosY\":58.1826172,\"PosZ\":15.9761353,\"Heading\":-2.96671629},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003602,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"F'zhumii\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-14.4198,\"PosY\":-14.3283081,\"PosZ\":20.7630348,\"Heading\":1.03866458},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003502,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-203.662231,\"PosY\":38.071167,\"PosZ\":16.0,\"Heading\":-1.27557254},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":1003503,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":3,\"Job\":0,\"Level\":1,\"Name\":\"\",\"CurrentHP\":1,\"MaxHP\":1,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":-204.211487,\"PosY\":39.16992,\"PosZ\":16.0,\"Heading\":-1.84401691},{\"CurrentWorldID\":18021,\"WorldID\":26994,\"WorldName\":\"\",\"BNpcID\":8,\"BNpcNameID\":1056964608,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":5,\"Job\":0,\"Level\":0,\"Name\":\"Aetheryte\",\"CurrentHP\":32759,\"MaxHP\":858665005,\"CurrentMP\":909130036,\"MaxMP\":10000,\"PosX\":-84.0314941,\"PosY\":0.0151977539,\"PosZ\":20.7674561,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":25669,\"WorldID\":25959,\"WorldName\":\"\",\"BNpcID\":49,\"BNpcNameID\":1817208173,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":5,\"Job\":0,\"Level\":0,\"Name\":\"Aethernet Shard\",\"CurrentHP\":32759,\"MaxHP\":1684960623,\"CurrentMP\":1735549268,\"MaxMP\":10000,\"PosX\":-213.611084,\"PosY\":51.80432,\"PosZ\":16.7391357,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":26222,\"WorldID\":2415,\"WorldName\":\"\",\"BNpcID\":44,\"BNpcNameID\":1937077345,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":5,\"Job\":0,\"Level\":0,\"Name\":\"Aethernet Shard\",\"CurrentHP\":32759,\"MaxHP\":1918984820,\"CurrentMP\":1851877443,\"MaxMP\":10000,\"PosX\":-179.40033,\"PosY\":182.970947,\"PosZ\":4.80651855,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":24933,\"WorldID\":2413,\"WorldName\":\"\",\"BNpcID\":2000402,\"BNpcNameID\":1735347315,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Market Board\",\"CurrentHP\":32759,\"MaxHP\":1208618288,\"CurrentMP\":1633842287,\"MaxMP\":10000,\"PosX\":-123.460815,\"PosY\":10.1166992,\"PosZ\":17.9903564,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":28271,\"WorldID\":29253,\"WorldName\":\"\",\"BNpcID\":2000402,\"BNpcNameID\":1275727152,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Market Board\",\"CurrentHP\":32759,\"MaxHP\":1131701587,\"CurrentMP\":1735287144,\"MaxMP\":10000,\"PosX\":-118.974609,\"PosY\":20.9506226,\"PosZ\":17.9903564,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":13364,\"WorldID\":3384,\"WorldName\":\"\",\"BNpcID\":2000401,\"BNpcNameID\":859060022,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Summoning Bell\",\"CurrentHP\":32759,\"MaxHP\":822703208,\"CurrentMP\":1208618292,\"MaxMP\":10000,\"PosX\":-123.888062,\"PosY\":21.4694214,\"PosZ\":17.9903564,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":11335,\"WorldID\":47036,\"WorldName\":\"\",\"BNpcID\":2000401,\"BNpcNameID\":4240164066,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Summoning Bell\",\"CurrentHP\":32759,\"MaxHP\":1918984820,\"CurrentMP\":1936683587,\"MaxMP\":10000,\"PosX\":-149.279053,\"PosY\":20.553894,\"PosZ\":18.20398,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":12656,\"WorldID\":31219,\"WorldName\":\"\",\"BNpcID\":2001578,\"BNpcNameID\":4128494095,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Guttering Streetlamp\",\"CurrentHP\":32759,\"MaxHP\":1952999273,\"CurrentMP\":158365264,\"MaxMP\":10000,\"PosX\":-65.6901245,\"PosY\":17.7766724,\"PosZ\":20.6759033,\"Heading\":-2.278859},{\"CurrentWorldID\":60526,\"WorldID\":9357,\"WorldName\":\"\",\"BNpcID\":2000402,\"BNpcNameID\":923326680,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Market Board\",\"CurrentHP\":32759,\"MaxHP\":1766090100,\"CurrentMP\":1346924659,\"MaxMP\":10000,\"PosX\":-166.979553,\"PosY\":28.4580078,\"PosZ\":17.9903564,\"Heading\":-4.79198352E-05},{\"CurrentWorldID\":38322,\"WorldID\":16980,\"WorldName\":\"\",\"BNpcID\":2001568,\"BNpcNameID\":3057433854,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Peculiar Herb\",\"CurrentHP\":32759,\"MaxHP\":1817208173,\"CurrentMP\":1130722401,\"MaxMP\":10000,\"PosX\":-44.7242432,\"PosY\":-0.5036011,\"PosZ\":20.0045166,\"Heading\":-0.286427528},{\"CurrentWorldID\":6382,\"WorldID\":2617,\"WorldName\":\"\",\"BNpcID\":2001572,\"BNpcNameID\":2037660840,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Guttering Streetlamp\",\"CurrentHP\":32759,\"MaxHP\":2035576904,\"CurrentMP\":1816225136,\"MaxMP\":10000,\"PosX\":-65.38495,\"PosY\":54.3983154,\"PosZ\":21.5609131,\"Heading\":-1.98270035},{\"CurrentWorldID\":8304,\"WorldID\":499,\"WorldName\":\"\",\"BNpcID\":2001567,\"BNpcNameID\":231861272,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Peculiar Herb\",\"CurrentHP\":32759,\"MaxHP\":158952805,\"CurrentMP\":809054772,\"MaxMP\":10000,\"PosX\":-30.47229,\"PosY\":0.8086548,\"PosZ\":19.97406,\"Heading\":2.428137},{\"CurrentWorldID\":37522,\"WorldID\":12356,\"WorldName\":\"\",\"BNpcID\":2001573,\"BNpcNameID\":1718939247,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Guttering Streetlamp\",\"CurrentHP\":32759,\"MaxHP\":1315268452,\"CurrentMP\":873030512,\"MaxMP\":10000,\"PosX\":-84.3977051,\"PosY\":81.16272,\"PosZ\":21.4998779,\"Heading\":1.19168162},{\"CurrentWorldID\":56489,\"WorldID\":14128,\"WorldName\":\"\",\"BNpcID\":2001566,\"BNpcNameID\":1450203579,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":0,\"Level\":0,\"Name\":\"Peculiar Herb\",\"CurrentHP\":32759,\"MaxHP\":1817208173,\"CurrentMP\":1315271777,\"MaxMP\":10000,\"PosX\":-18.32611,\"PosY\":1.72418213,\"PosZ\":20.0045166,\"Heading\":-1.75202441},{\"CurrentWorldID\":19662,\"WorldID\":37851,\"WorldName\":\"\",\"BNpcID\":2001565,\"BNpcNameID\":3253406825,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":7,\"Job\":114,\"Level\":105,\"Name\":\"Peculiar Herb\",\"CurrentHP\":873031015,\"MaxHP\":892876594,\"CurrentMP\":959984696,\"MaxMP\":10000,\"PosX\":-11.39856,\"PosY\":6.63763428,\"PosZ\":19.97406,\"Heading\":2.54807687}],\"rseq\":0}";
		String inStr = inStrPart1 + inStrPart2;
		byte[] inBytes = inStr.getBytes(StandardCharsets.UTF_8);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		GZIPOutputStream gzip = new GZIPOutputStream(baos);
		gzip.write(inBytes);
		gzip.close();
		byte[] outBytes = baos.toByteArray();
		String outStr = baos.toString(StandardCharsets.UTF_8);

		log.info("Bytes: {} -> {}", inBytes.length, outBytes.length);
		log.info("UTF-8: {} -> {}", inStr.length(), outStr.length());


	}

	@Test
	public void testCompression3() throws IOException {
		String inStr = "{\"combatants\":[{\"CurrentWorldID\":65,\"WorldID\":65,\"WorldName\":\"Midgardsormr\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":275160354,\"OwnerID\":0,\"type\":1,\"Job\":24,\"Level\":80,\"Name\":\"Wynn Dohz\",\"CurrentHP\":122634,\"MaxHP\":122634,\"CurrentMP\":10000,\"MaxMP\":10000,\"PosX\":623.9719,\"PosY\":95.42591,\"PosZ\":23.840683,\"Heading\":3.141497},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905555,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3381,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905549,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":603437,\"MaxHP\":603437,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.8749,\"PosY\":66.42804,\"PosZ\":23.75934,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":0,\"BNpcNameID\":0,\"PartyType\":0,\"ID\":3758096384,\"OwnerID\":0,\"type\":8,\"Job\":0,\"Level\":0,\"Name\":\"\",\"CurrentHP\":0,\"MaxHP\":0,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.8749,\"PosY\":66.42804,\"PosZ\":23.75934,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905556,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905558,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905559,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905551,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905552,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905553,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905554,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905557,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3384,\"BNpcNameID\":887,\"PartyType\":0,\"ID\":1073905550,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Odin\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":622.622,\"PosY\":61.74554,\"PosZ\":23.65141,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3383,\"BNpcNameID\":1858,\"PartyType\":0,\"ID\":1073905544,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Gungnir\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":618.8301,\"PosY\":65.79023,\"PosZ\":23.58328,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3383,\"BNpcNameID\":1858,\"PartyType\":0,\"ID\":1073905545,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Gungnir\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":618.8301,\"PosY\":65.79023,\"PosZ\":23.58328,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3383,\"BNpcNameID\":1858,\"PartyType\":0,\"ID\":1073905546,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Gungnir\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":618.8301,\"PosY\":65.79023,\"PosZ\":23.58328,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3383,\"BNpcNameID\":1858,\"PartyType\":0,\"ID\":1073905547,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Gungnir\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":618.8301,\"PosY\":65.79023,\"PosZ\":23.58328,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":0,\"WorldID\":0,\"WorldName\":\"\",\"BNpcID\":3383,\"BNpcNameID\":1858,\"PartyType\":0,\"ID\":1073905548,\"OwnerID\":0,\"type\":2,\"Job\":0,\"Level\":50,\"Name\":\"Gungnir\",\"CurrentHP\":2778,\"MaxHP\":2778,\"CurrentMP\":0,\"MaxMP\":10000,\"PosX\":618.8301,\"PosY\":65.79023,\"PosZ\":23.58328,\"Heading\":-4.79221344E-05},{\"CurrentWorldID\":46883,\"WorldID\":61658,\"WorldName\":\"\",\"BNpcID\":2000182,\"BNpcNameID\":3844922158,\"PartyType\":0,\"ID\":1073905543,\"OwnerID\":0,\"type\":7,\"Job\":189,\"Level\":213,\"Name\":\"\",\"CurrentHP\":1515434228,\"MaxHP\":2524395700,\"CurrentMP\":3991882399,\"MaxMP\":10000,\"PosX\":623.0,\"PosY\":94.50564,\"PosZ\":23.87231,\"Heading\":0.0},{\"CurrentWorldID\":18753,\"WorldID\":13553,\"WorldName\":\"\",\"BNpcID\":2000139,\"BNpcNameID\":1863677006,\"PartyType\":0,\"ID\":1073905542,\"OwnerID\":0,\"type\":7,\"Job\":59,\"Level\":93,\"Name\":\"Exit\",\"CurrentHP\":1849490117,\"MaxHP\":3762351342,\"CurrentMP\":2845777030,\"MaxMP\":10000,\"PosX\":623.0,\"PosY\":61.95618,\"PosZ\":23.65626,\"Heading\":0.0}],\"rseq\":0}";
		byte[] inBytes = inStr.getBytes(StandardCharsets.UTF_8);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		GZIPOutputStream gzip = new GZIPOutputStream(baos);
		gzip.write(inBytes);
		gzip.close();
		byte[] outBytes = baos.toByteArray();
		String outStr = baos.toString(StandardCharsets.UTF_8);

		log.info("Bytes: {} -> {}", inBytes.length, outBytes.length);
		log.info("UTF-8: {} -> {}", inStr.length(), outStr.length());
	}

	@Test
	public void testJShell() {
		try (JShell shell = JShell.builder().executionEngine(new LocalExecutionControlProvider(), Collections.emptyMap()).build()) {
			shell.eval("Foo");
		}

	}

	@Test
	public void timeJShell() {
		int count = 1_000;
		timeIt("JShell", () -> {
			for (int i = 0; i < count; i++) {
				testJShell();
			}
		});
	}

	@Test
	public void timeJShellEval() {
		try (JShell shell = JShell.builder().executionEngine(new LocalExecutionControlProvider(), Collections.emptyMap()).build()) {
			timeIt("JShell Init", () -> {
				shell.eval("Foo");
			});
			int count = 1_00;
			timeIt("JShell Loop", () -> {
				for (int i = 0; i < count; i++) {
					shell.eval("123 + 456");
				}
			});
		}
	}

	@Test
	public void timeBshNewInterp() {
		int count = 1_000;
		timeIt("Bsh Loop, new interpreter on every iteration", () -> {
			for (int i = 0; i < count; i++) {
				Interpreter shell = new Interpreter();
				try {
					shell.set("i", i);
					shell.eval("i + 456");
				}
				catch (EvalError e) {
					log.error("Error", e);
				}
			}
		});
	}

	@Test
	public void timeBshSameInterp() {
		int count = 100_000;
		timeIt("Bsh Loop", () -> {
			Interpreter shell = new Interpreter();
			for (int i = 0; i < count; i++) {
				try {
					shell.set("i", i);
					shell.eval("123 + 456");
					shell.unset("i");
				}
				catch (EvalError e) {
					log.error("Error", e);
				}
			}
		});
	}

	@Test
	public void timeGroovySameInterp() {
		int count = 1_000;
		timeIt("Groovy Loop", () -> {
			GroovyShell shell = new GroovyShell();
			for (int i = 0; i < count; i++) {
				shell.setVariable("i", 456);
				shell.evaluate("i + 456");
				shell.removeVariable("i");
			}
		});
	}

	@Test
	public void timeGroovyClosure() {
		int count = 100_000;
		timeIt("Groovy Loop, Compiled Closure", () -> {
			GroovyShell shell = new GroovyShell();
			Script script = shell.parse("i + 456");
			for (int i = 0; i < count; i++) {
				shell.setVariable("i", i);
				Object run = script.run();
				shell.removeVariable("i");
			}
		});
	}

	@Test
	public void timeGroovyClosure2() {
		int count = 100_000;
		timeIt("Groovy Loop, Compiled Closure", () -> {
			GroovyShell shell = new GroovyShell();
			Script script = shell.parse("s.toUpperCase();");
			for (int i = 0; i < count; i++) {
				shell.setVariable("s", "FooBar");
				Object run = script.run();
				shell.removeVariable("s");
			}
		});
	}

	@Test
	public void timeGroovyClosure3() {
		int count = 100_000;
		timeIt("Groovy Loop, Compiled Closure", () -> {
			GroovyShell shell = new GroovyShell();
			GString string = GString.EMPTY.plus("${s.toUpperCase()}");
//			Script script = shell.parse("s.toUpperCase();");
			for (int i = 0; i < count; i++) {
				string.setProperty("s", "FooBar");
//				string.setVariable("s", "FooBar");

				Object run = string.toString();
				shell.removeVariable("s");
			}
		});
	}

	@Test
	public void timeReplacements() {
		int count = 1_000;
		timeIt("Callout Initial", CalloutTests::testReplacementsAdvanced);
		timeIt("Advanced Callout Replacement", () -> {
			for (int i = 0; i < count; i++) {
				CalloutTests.testReplacementsAdvanced();
			}
		});
	}

	@Test
	public void testScriptEngine() {
		ScriptEngineManager sem = new ScriptEngineManager();
		sem.getEngineFactories();
	}
}
